# -*- coding: utf-8 -*-
from setuphelpers import *
import json
import platform

uninstallkey = []

# Declaring specific app values (TO CHANGE)
bin_name_string = "rocketchat-setup-%s.exe"
uninstallkey_app = "9b73a9fb-f1d5-59ee-b41e-e1dd393a748a"
silent_args = "/S /allusers /disableAutoUpdates"
dics = "dictionaries"
dictionaries_dir = makepath(programfiles, "Rocket.Chat", "resources", "dictionaries")


def install():
    # Specific app values
    package_version = control.version.split("-", 1)[0]

    # Installing the package
    install_exe_if_needed(
        bin_name_string % package_version,
        silentflags=silent_args,
        key=uninstallkey_app,
        min_version=package_version,
    )

    # Adding language Dictionaries
    for copy_dics in glob.glob(makepath(dics, "*")):
        filecopyto(copy_dics, dictionaries_dir)

    # Deleting desktop shortcut
    remove_desktop_shortcut("Rocket.Chat")


def update_package():
    print("Download/Update package content from upstream binary sources")

    # Getting proxy informations from WAPT settings
    proxy = {}
    if platform.system() == "Windows" and isfile(makepath(user_local_appdata(), "waptconsole", "waptconsole.ini")):
        proxywapt = inifile_readstring(
            makepath(user_local_appdata(), "waptconsole", "waptconsole.ini"),
            "global",
            "http_proxy",
        )
        if proxywapt:
            proxy = {"http": proxywapt, "https": proxywapt}

    # Specific app values
    app_name = control.name
    url_api = "https://api.github.com/repos/RocketChat/Rocket.Chat.Electron/releases/latest"
    bin_end = bin_name_string.split("%s")[-1]

    # Getting latest informations from Github API
    json_load = json.loads(wgets(url_api, proxies=proxy))
    for download in json_load["assets"]:
        if download["browser_download_url"].endswith(bin_end):
            url_dl = download["browser_download_url"]
            break

    version = (url_dl.split("-")[-1]).replace(bin_end, "")
    latest_bin = bin_name_string % version

    print("Latest " + app_name + " version is: " + version)
    print("Download url is: " + url_dl)

    # Downloading latest binaries
    if not isfile(latest_bin):
        print("Downloading: " + latest_bin)
        wget(url_dl, latest_bin, proxies=proxy)

        # Checking version from file
        version_from_file = get_file_properties(latest_bin)["ProductVersion"]
        if version != version_from_file and version_from_file != "":
            version = version_from_file
            old_latest_bin = latest_bin
            latest_bin = bin_name_string % version
            if isfile(latest_bin):
                remove_file(latest_bin)
            os.rename(old_latest_bin, latest_bin)

        # Changing version of the package
        pe = PackageEntry().load_control_from_wapt(".")
        pe.version = "%s-%s" % (version, int(pe.version.split("-", 1)[1]) + 1)
        pe.save_control_to_wapt(".")
        print("Changing version to " + pe.version + " in WAPT\\control")
        print("Update package done. You can now build-upload your package")
    else:
        print("This package is already up-to-date")

    # Downloading dictionaries
    # FR
    dic_fr = "fr_FR.dic"
    aff_fr = "fr_FR.aff"
    # Most recent alternative: https://grammalecte.net/download.php?prj=fr
    url_dl_dic_fr = "https://raw.githubusercontent.com/LibreOffice/dictionaries/master/fr_FR/fr.aff"
    url_dl_aff_fr = "https://raw.githubusercontent.com/LibreOffice/dictionaries/master/fr_FR/fr.dic"
    latest_dic_fr = r"%s\%s" % (dics, dic_fr)
    latest_aff_fr = r"%s\%s" % (dics, aff_fr)
    if not isfile(latest_dic_fr):
        print("Downloading: " + latest_dic_fr)
        wget(url_dl_dic_fr, latest_dic_fr, proxies=proxy)
    if not isfile(latest_aff_fr):
        print("Downloading: " + latest_aff_fr)
        wget(url_dl_aff_fr, latest_aff_fr, proxies=proxy)

    # Deleting outdated binaries
    for bin_in_dir in glob.glob("*.exe") or glob.glob("*.msi"):
        if bin_in_dir != latest_bin:
            print("Outdated binary: " + bin_in_dir + " Deleted")
            remove_file(bin_in_dir)
