# -*- coding: utf-8 -*-
from setuphelpers import *
import platform

uninstallkey = []

# Defining variables
bin_name_string = "StarLeaf-%s-Windows.exe"
silent_args = "/S"
app_uninstallkey = ""
app_dir = makepath(programfiles, "StarLeaf")


def install():
    # Initializing variables
    package_version = control.version.split("-", 1)[0]
    bin_name = bin_name_string % package_version
    bin_path = makepath(app_dir, bin_name)
    processes_to_kill = control.impacted_process.split(",")

    # Installing the package
    if not isdir(app_dir):
        print("Creating %s directory" % app_dir)
        mkdirs(app_dir)
    if isfile(bin_path):
        remove_file(bin_path)
    print("Copying %s to %s" % (bin_name, app_dir))
    filecopyto(bin_name, app_dir)


def uninstall():
    # Initializing variables
    processes_to_kill = control.impacted_process.split(",")

    # Uninstalling the package
    killalltasks(processes_to_kill)
    if isdir(app_dir):
        remove_tree(app_dir)


def session_setup():
    # Initializing variables
    package_version = control.version.split("-")[0]
    bin_name = bin_name_string % package_version
    bin_path = makepath(app_dir, bin_name)
    app_name = control.name
    processes_to_kill = control.impacted_process.split(",")
    # "C:\Users\username\AppData\Local\StarLeaf\StarLeaf\StarLeaf.exe"
    user_app_dir = makepath(user_local_appdata, "StarLeaf", "StarLeaf")
    user_app_bin = makepath(user_app_dir, "StarLeaf.exe")
    user_app_uninst_path = makepath(user_app_dir, "Misc", "Uninstall.exe")
    user_app_bin_version = get_version_from_binary(user_app_bin)
    # user_app_reg_version = registry_readstring(HKEY_CURRENT_USER,r'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\StarLeaf','DisplayVersion')

    # Installing the package in user environment
    if user_app_bin_version == "" or package_version > user_app_bin_version:
        """# Uninstall may be required a day
        killalltasks(processes_to_kill)
        run('"%s" %s' % (user_app_uninst_path, silent_args))"""
        print("Installing: %s in user environment" % (bin_name))
        """ killalltasks(processes_to_kill)
        print('"%s" %s' % (bin_path, silent_args))
        run('"%s" %s' % (bin_path, silent_args)) """
        install_exe_if_needed(
            bin_path,
            silentflags=silent_args,
            key=app_uninstallkey,
            killbefore=processes_to_kill,
            min_version=package_version,
        )
    else:
        print("OK: %s is already installed, or %s is a newer version than %s." % (app_name, user_app_bin_version, package_version))


def update_package():
    # Initializing variables
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    url_dl = "https://dl.starleaf.com/app/StarLeaf-Installer-Windows.exe"
    bin_temp = "StarLeaf-Installer-Windows.exe"
    version = "1.0"
    latest_bin = bin_temp

    print("Download url is: %s" % url_dl)

    # Downloading latest binaries
    if not isfile(latest_bin):
        print("Downloading: %s" % latest_bin)
        wget(url_dl, latest_bin, proxies=proxies)

        # Checking version from file
        version_from_file = get_version_from_binary(latest_bin)
        if version != version_from_file:
            os.rename(latest_bin, bin_name_string % version_from_file)
            version = version_from_file

        # Changing version of the package
        control.version = "%s-%s" % (version, int(control.version.split("-")[-1]) + 1)
        control.save_control_to_wapt()
        print("Changing package version to: %s in WAPT\\control" % control.version)

    # Deleting outdated binaries
    remove_outdated_binaries(latest_bin)


def get_proxies():
    if platform.python_version_tuple()[0] == "3":
        from urllib.request import getproxies
    else:
        from urllib import getproxies
    return getproxies()


def get_proxies_from_wapt_console():
    proxies = {}
    if platform.system() == "Windows":
        waptconsole_ini_path = makepath(user_local_appdata(), "waptconsole", "waptconsole.ini")
    else:
        waptconsole_ini_path = makepath(user_home_directory(), ".config", "waptconsole", "waptconsole.ini")
    if isfile(waptconsole_ini_path):
        proxy_wapt = inifile_readstring(waptconsole_ini_path, "global", "http_proxy")
        if proxy_wapt:
            proxies = {"http": proxy_wapt, "https": proxy_wapt}
    return proxies


def get_version_from_binary(filename):
    if filename.endswith(".msi"):
        return get_msi_properties(filename)["ProductVersion"]
    else:
        return get_file_properties(filename)["ProductVersion"]


def remove_outdated_binaries(
    version,
    list_extensions=["exe", "msi", "deb", "rpm", "dmg", "pkg"],
    list_filename_contain=None,
):
    if type(list_extensions) != list:
        list_extensions = [list_extensions]
    if list_filename_contain:
        if type(list_filename_contain) != list:
            list_filename_contain = [list_filename_contain]
    list_extensions = ["." + ext for ext in list_extensions if ext[0] != "."]
    for file_ext in list_extensions:
        for bin_in_dir in glob.glob("*%s" % file_ext):
            if not version in bin_in_dir:
                remove_file(bin_in_dir)
            if list_filename_contain:
                for filename_contain in list_filename_contain:
                    if not filename_contain in bin_in_dir:
                        remove_file(bin_in_dir)


def bs_find_all(url, element, attribute=None, value=None, headers=None, proxies=None, features="html.parser", **kwargs):
    """ "You may need to use a header for some websites. For example: headers={'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:78.0) Gecko/20100101 Firefox/78.0'}"""
    import requests

    page = requests.get(url, proxies=proxies, headers=headers, **kwargs).text
    try:
        import bs4 as BeautifulSoup

        soup = BeautifulSoup.BeautifulSoup(page, features=features)
    except:
        import BeautifulSoup

        soup = BeautifulSoup.BeautifulSoup(page)
    if value:
        return soup.findAll(element, {attribute: value})
    else:
        return soup.findAll(element)
