def session_setup():
    # Specific app values
    user_conf_dir = makepath(application_data, "Microsoft", "Teams")
    json_config = makepath(user_conf_dir, "desktop-config.json")
    json_settings = makepath(user_conf_dir, "settings.json")

    user_app_dir = makepath(user_local_appdata, "Microsoft", "Teams")
    user_app_path = makepath(user_app_dir, "current", "Teams.exe")
    user_app_updater = makepath(user_app_dir, "Update.exe")
    user_app_version = registry_readstring(
        HKEY_CURRENT_USER,
        r"Software\Microsoft\Windows\CurrentVersion\Uninstall\Teams",
        "DisplayVersion",
    )

    user_app_progdata_dir = makepath(systemdrive, "ProgramData", get_current_user(), "Microsoft", "Teams")
    user_app_progdata_path = makepath(user_app_progdata_dir, "current", "Teams.exe")
    user_app_progdata_updater = makepath(user_app_progdata_dir, "Update.exe")

    package_version = control.version.split("-")[0]
    user_silent_args_uninst = "--uninstall -s"

    # Killing app before making changes
    killalltasks("Teams.exe")

    # Prevent stuck installation because of user environment install (procedure: https://docs.microsoft.com/microsoftteams/msi-deployment)
    if Version(user_app_version) < Version(package_version):
        if isfile(user_app_path):
            print("Uninstalling Microsoft Teams from user environment")
            run('"%s" %s' % (user_app_updater, user_silent_args_uninst))
            if isdir(user_app_dir):
                remove_tree(user_app_dir)
        if isfile(user_app_progdata_path):
            print("Uninstalling Microsoft Teams from user ProgramData environment")
            run('"%s" %s' % (user_app_progdata_updater, user_silent_args_uninst))
            if isdir(user_app_progdata_dir):
                remove_tree(user_app_progdata_dir)

    if reg_value_exists(
        HKEY_CURRENT_USER,
        r"Software\Microsoft\Office\Teams",
        "PreventInstallationFromMsi",
    ):
        registry_delete(
            HKEY_CURRENT_USER,
            r"Software\Microsoft\Office\Teams",
            "PreventInstallationFromMsi",
        )

    # Define user settings (procedure: https://techcommunity.microsoft.com/t5/microsoft-teams/how-to-disable-check-for-updates-in-microsoft-teams/m-p/54644)
    print("Setting up user settings")
    if not isdir(user_conf_dir):
        mkdirs(user_conf_dir)

    # desktop-config.json
    if isfile(json_config):
        json_config_content = load_json(json_config)
    else:
        json_config_content = json.loads('{"appPreferenceSettings":{}}')

    json_config_content["appPreferenceSettings"]["openAtLogin"] = False
    json_config_content["appPreferenceSettings"]["openAsHidden"] = False
    json_config_content["appPreferenceSettings"]["isForeGround"] = False
    write_json(json_config, json_config_content)

    # settings.json
    if isfile(json_settings):
        json_settings_content = load_json(json_settings)
    else:
        json_settings_content = json.loads('{"baseSettings":{},"initializedSettings":{}}')

    json_settings_content["baseSettings"]["enableBlockSilentUpdate"] = True
    json_settings_content["initializedSettings"]["disableAllUsersAutoUpdate"] = True
    write_json(json_settings, json_settings_content)
