# -*- coding: utf-8 -*-
from setuphelpers import *

"""
Usable WAPT package functions: install(), uninstall(), session_setup(), audit(), update_package()

https://docs.microsoft.com/powershell/scripting/install/installing-windows-powershell
https://docs.microsoft.com/powershell/scripting/windows-powershell/install/installing-windows-powershell?view=powershell-7.1#upgrading-existing-windows-powershell
https://www.microsoft.com/download/details.aspx?id=54616

"""
# Declaring global variables - Warnings: 1) WAPT context is only available in package functions; 2) Global variables are not persistent between calls
kbs_dict = [
    {
        "dl_url": "https://download.microsoft.com/download/6/F/5/6F5FF66C-6775-42B0-86C4-47D41F2DA187/Win7AndW2K8R2-KB3191566-x64.zip",
        "kb_name": "Win7AndW2K8R2-KB3191566-x64.msu",
        "win_vers": WindowsVersions.Windows7,
        "arch": "x64",
    },
    {
        "dl_url": "https://download.microsoft.com/download/6/F/5/6F5FF66C-6775-42B0-86C4-47D41F2DA187/Win7-KB3191566-x86.zip",
        "kb_name": "Win7-KB3191566-x86.msu",
        "win_vers": WindowsVersions.Windows7,
        "arch": "x86",
    },
    {
        "dl_url": "https://download.microsoft.com/download/6/F/5/6F5FF66C-6775-42B0-86C4-47D41F2DA187/W2K12-KB3191565-x64.msu",
        "kb_name": "W2K12-KB3191565-x64.msu",
        "win_vers": WindowsVersions.Windows81,
        "arch": "x64",
    },
    {
        "dl_url": "https://download.microsoft.com/download/6/F/5/6F5FF66C-6775-42B0-86C4-47D41F2DA187/Win8.1AndW2K12R2-KB3191564-x64.msu",
        "kb_name": "Win8.1AndW2K12R2-KB3191564-x64.msu",
        "win_vers": WindowsVersions.WindowsServer2012,
        "arch": "x64",
    },
    {
        "dl_url": "https://download.microsoft.com/download/6/F/5/6F5FF66C-6775-42B0-86C4-47D41F2DA187/Win8.1-KB3191564-x86.msu",
        "kb_name": "Win8.1-KB3191564-x86.msu",
        "win_vers": WindowsVersions.Windows81,
        "arch": "x86",
    },
]


def install():
    # Declaring local variables
    if iswin64():
        arch = "x64"
    else:
        arch = "x86"
    patched = False

    # Installing Windows KBs
    for kb in kbs_dict:
        kb_file = kb["kb_name"]
        kb_name = kb_file.split("-")[1].upper()
        kb_nb = kb_name.replace("KB", "")
        if kb["arch"] == arch:
            if kb["win_vers"] == windows_version(members_count=2):
                with EnsureWUAUServRunning():
                    if not is_kb_installed(kb_nb):
                        print("Installing: {}".format(kb_file))
                        run(
                            'wusa.exe "{}" /quiet /norestart'.format(kb_file),
                            accept_returncodes=[0, 3010, 2359302, -2145124329],
                            timeout=3600,
                        )
                        if not is_kb_installed(kb_nb):
                            error("{} installation do not complete".format(kb_name))
                    else:
                        print("{} is already installed".format(kb_name))
                    patched = True
    if not patched:
        print("INFO: No need to patch this PC")


def update_package():
    # Declaring local variables
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()

    # Downloading latest binaries
    for dl_kb in kbs_dict:
        dl_kb_file = dl_kb["dl_url"].split("/")[-1]
        if not isfile(dl_kb_file):
            wget(dl_kb["dl_url"], dl_kb_file, proxies=proxies)

    # Extracting
    for zip_in_dir in glob.glob("*.zip"):
        print("Extracting: " + zip_in_dir)
        unzip(zip_in_dir, filenames="*.msu", ignore_basename=True)
        remove_file(zip_in_dir)
