def remove_drivers_by_manufacturer(manufacturer_list=[]):
    r"""This function will check existing drivers by manufacturer and delete them using pnputil

    Args:
        manufacturer_list : a list strings determining the drivers to be deleted

    Returns:
        driver_dict_list : a list of dictionnaires of the deleted drivers

    The dictionnary's key are as follow:
              Key                           Example
    0    "Nom publiÚá"                   oem0.inf
    1    "Nom dÆorigineá"                xennet.inf
    2    "Nom du fournisseurá"           XCP-ng
    3    "Nom de la classeá"             Cartes rÚseau
    4    "GUID de la classeá"            {4d36e972-e325-11ce-bfc1-08002be10318}
    5    "Version du piloteá"            06/26/2019 8.2.2.0
    6    "Nom du signataireá"            Vates

        The "à", "Ú" and "Æ" are not an error, they are part of the keys(in french at least)

    """
    result = ensure_unicode(run("pnputil /enum-drivers"))
    result = result.split("\r\n\r\n")  # Splitting drivers sections
    driver_dict_list = []

    # Creation of the list of dico
    for drivers in result:
        properties_dict = {}
        for properties in drivers.split("\r\n"):
            if ":" in properties:
                properties_dict[properties.split(":")[0]] = properties.split(":")[1].strip()
        if not len(list(properties_dict)) == 7:
            continue
        for manufacturer in manufacturer_list:
            if manufacturer == list(properties_dict.values())[2]:
                driver_dict_list.append(properties_dict)

    # add deletion of drivers
    for driver_dict in driver_dict_list:
        driver_dict = list(driver_dict.values())
        print("Removing: %s, oem n°: %s" % (driver_dict[1], driver_dict[0]))
        run(f"pnputil /delete-driver {driver_dict[0]} /uninstall /force /reboot")

    return driver_dict_list
