#!waptpython
# -*- coding: utf-8 -*-
from setuphelpers import *
import waptpackage
import os
import json
import requests
import tempfile
from concurrent.futures import ThreadPoolExecutor as Pool
import waptguihelper


############################# Functions #############################


def convert_control_path_to_dir(fp):
    if fp.endswith("%sWAPT%scontrol" % (os.sep, os.sep)):
        fp = fp.rsplit("%sWAPT%scontrol" % (os.sep, os.sep))[0]
    return fp


def get_control_dict(pkg_dir):
    pkg_dir = convert_control_path_to_dir(pkg_dir)
    control_dict = waptpackage.PackageEntry().load_control_from_wapt(pkg_dir).as_dict()
    return control_dict


############################# Variables #############################
config = json_load_file("config.json")
git_servers = config["git_servers"]
gitea_url = waptguihelper.combo_dialog("Please select git server", "", git_servers)
git_orgs = config[gitea_url]["orgs"]
gitea_org = waptguihelper.combo_dialog("Please select git org", "", git_orgs)
# gitea_url = "git.tranquil.it"
# gitea_org = "wapt-education"
gitea_access_token = config[gitea_url]["access_token"]
# org_dir = makepath(rf"C:\waptdev-private\{gitea_org}")
org_dir = waptguihelper.directory_dialog("Please select dir where to git pull")
skip_repos_until = ""
repos_skipped = False
repos_checked = 0
package_template_path = makepath(config["package-tools"], "package-template")
log_failed_pull_file = "failed_pull_package_list.txt"
failed_pull_package_list = []
excluse_f_path = makepath(tempfile.gettempdir(), "excludedfileslist.txt")
launch_update_package = False
allow_to_remove_failed_pull = False
# if waptguihelper.message_dialog("multi-pull", "remove failed pull repo?", waptguihelper.ID_YES) == waptguihelper.ID_YES:
#     allow_to_remove_failed_pull = True
# else:
#     allow_to_remove_failed_pull = False
# if waptguihelper.message_dialog("multi-pull", "Run update-package-sources?", waptguihelper.ID_YES) == waptguihelper.ID_YES:
#     launch_update_package = True
# else:
#     launch_update_package = False


repo_result = {}


nb_cores = os.cpu_count()

############################# Code      #############################
if not isdir(org_dir):
    mkdirs(org_dir)
elif allow_to_remove_failed_pull and isfile(makepath(org_dir, log_failed_pull_file)):
    with open(makepath(org_dir, log_failed_pull_file), "r") as f:
        failed_repos = f.read().strip().split("\n")
    for failed_repo in failed_repos:
        failed_repo_path = makepath(org_dir, failed_repo.split("tis-", 1)[1])
        if isdir(failed_repo_path):
            print("Removing: %s" % failed_repo_path)
            remove_tree(failed_repo_path)

with open(excluse_f_path, "w") as f:
    f.write(".gitignore\\\n")

# Script
org_repos = f"https://{gitea_url}/api/v1/orgs/{gitea_org}/repos?access_token={gitea_access_token}"
org_repos_result = json.loads(wgets(org_repos))
ordered_org_repos = sorted(org_repos_result, key=lambda d: d["name"], reverse=False)


def update_repository(repo_name):
    repo_result[repo_name] = ""

    os.chdir(org_dir)
    # Committing
    repo_dir = makepath(os.getcwd(), repo_name.split("tis-", 1)[1])
    org_repos = f"https://{gitea_url}/api/v1/orgs/{gitea_org}/repos?access_token={gitea_access_token}"
    api_url_repo = f"https://{gitea_url}/api/v1/repos/{gitea_org}/{repo_name}?access_token={gitea_access_token}"
    clone_url = f"https://{gitea_url}/{gitea_org}/{repo_name}.git"
    git_url = f"https://{gitea_url}/{gitea_org}/{repo_name}"
    git_url_logged = f"https://{gitea_access_token}@{gitea_url}/{gitea_org}/{repo_name}"
    api_url_create_repo = f"https://{gitea_url}/api/v1/org/{gitea_org}/repos?access_token={gitea_access_token}"
    answer_commit_files = None
    answer_create_repo = None

    print(repo_name)
    check_repo = requests.get(api_url_repo).status_code
    if check_repo == 200:
        if json.loads(wgets(api_url_repo))["archived"]:
            if isdir(repo_dir):
                repo_result[repo_name] += "Removing: %s" % repo_dir
                remove_tree(repo_dir)
            return f"{repo_name} archived"
    elif check_repo == 404:
        repo_result[repo_name] += "%s repo_name do not exist" % repo_name
    else:
        repo_result[repo_name] += "Unable to access repo_name or organization"
        error("Unable to access repo_name or organization")

    # Git Clone if needed
    try:
        if not isdir(repo_dir):
            run("git clone %s %s" % (git_url_logged, repo_dir), accept_returncodes=[0, 128])
            os.chdir(repo_dir)
            run("git remote set-url origin %s" % (git_url), accept_returncodes=[0, 128], cwd=repo_dir)
            if gitea_access_token in str(run_notfatal("git config --get remote.origin.url", cwd=repo_dir)).strip():
                error("gitea_access_token found in remote.origin.url")
        else:
            os.chdir(repo_dir)
            pull_result = run("git pull %s --rebase --autostash" % git_url_logged, accept_returncodes=[0, 128])
            repo_result[repo_name] += pull_result
        # r = run("git config --global --add safe.directory %s" % repo_dir.replace("\\", "/"), accept_returncodes=[0, 128])
        # repo_result[repo_name] += r
    except Exception as e:
        print(e)
        failed_pull_package_list.append(repo_name)

    # run(r'xcopy /r /d /i /s /y /exclude:%s "%s" "%s"' % (excluse_f_path, package_template_path, repo_dir))

    if launch_update_package:
        dir_list = glob.glob("*/")
        # [dir_list.remove(a) for a in ["WAPT\\"] if a]
        for os_dir in dir_list:
            os_dir_path = makepath(repo_dir, os_dir)
            try:
                repo_result[repo_name] += run(f'wapt-get update-package-sources "{os_dir_path}" --no-ide')
            except:
                repo_result[repo_name] += f'wapt-get update-package-sources "{os_dir_path}" failed'

    return repo_name


results = {}

import time
import signal


def signal_handler(*args, **kwargs):
    pool.shutdown(False)
    for repo_name, task in results.items():
        if not task.done:
            print(f"{repo_name} has been cancelled")


signal.signal(signal.SIGINT, signal_handler)

try:
    with Pool(max_workers=nb_cores) as pool:
        # results = pool.map(update_repository, ordered_org_repos)

        for repo in ordered_org_repos:
            time.sleep(0.1)
            repo_name = repo["name"]
            if repo_name.startswith("package-"):
                continue
            # Skip part
            if skip_repos_until:
                if repo_name == skip_repos_until:
                    repos_skipped = True
                else:
                    if not repos_skipped:
                        continue
            else:
                repos_skipped = True
            committed = False
            repos_checked += 1

            if repos_skipped:
                res = pool.submit(update_repository, repo_name)
                results[repo_name] = res  # res représente la variable qui permettra
                # d'accéder au résultat quand la fonction update sera terminée

        input("Enter to continue or ctrl-c to cancel")

        for repo_name, res in results.items():
            r = res.result()  # bloque le code en attente du résultat
            print(r, repo_result[r])

    log_failed_pull_path = org_dir + os.sep + log_failed_pull_file
    if failed_pull_package_list:
        # with open(log_failed_pull_path, "w") as output:
        #     output.write("[")
        #     for row in failed_pull_package_list:
        #         output.write('"' + str(row) + '",' + "\n")
        #     output.write("]")
        with open(log_failed_pull_path, "w") as output:
            for row in failed_pull_package_list:
                output.write(str(row) + "\n")
    else:
        if isfile(log_failed_pull_path):
            remove_file(log_failed_pull_path)
except KeyboardInterrupt as err:
    pool.shutdown(False)  # , cancel_futures=True)

    for repo_name, task in results.items():
        if not task.done:
            print(f"{repo_name} has been cancelled")
